#ifndef SD_CMD_H
#define SD_CMD_H

#include <stdint.h>

enum sd_errcodes {
	SD_OK = 0,
	SD_ERR = 1,
	SD_ERR_CRC = 2,
	SD_ERR_WRITE = 3,
};

enum sd_cardtype {
	SD_CARDTYPE_UNKNOWN = 0,
	SD_CARDTYPE_MMC,
	SD_CARDTYPE_SDV1,
	SD_CARDTYPE_SDv2,
	SD_CARDTYPE_SDHC,
};

struct sd_cardinfo {
	enum sd_cardtype cardtype;
	uint16_t blocksize;
	uint16_t READ_BL_LEN;		//length is computed as 2^READ_BL_LEN (9 = 512byte)
	uint32_t memory_capacity;
	uint32_t block_nbr;
};

/** \brief Initialisiert die SD Karte und bestimmt ihren Typ
	\return SD_OK oder SD_ERR; MMC wird nicht untersttzt -> immer SD_ERR
*/
enum sd_errcodes sd_init_card(void);

/** \brief gibt Karteninformationen preis
	\return Pointer auf \see struct sd_cardinfo
*/
struct sd_cardinfo* sd_get_cardinfo(void);
 
/**	\brief liest einen oder mehrere Blcke von der SD Karte
	\param buf Datenbuffer in den gelesen wird
	\param lba Blockadresse ab der gelesen werden soll
	\param block_count Anzahl der zu lesenden Blcke
	\param blocksize Gre eines Blocks auf der SD Karte
	\return SD_OK oder SD_ERR
*/
enum sd_errcodes 
sd_read_blocks(uint8_t *buf, uint32_t lba, uint32_t block_count, uint32_t blocksize);

/**	\brief schreibt einen oder mehrere Blcke auf die SD Karte
	\param buf Datenbuffer von dem gelesen wird
	\param lba Blockadresse ab der geschrieben werden soll
	\param block_count Anzahl der zu schreibenden Blcke
	\param blocksize Gre eines Blocks auf der SD Karte
	\return SD_OK oder SD_ERR
*/
enum sd_errcodes 
sd_write_blocks(const uint8_t *buf, uint32_t lba, uint32_t block_count, uint32_t blocksize);

#endif//SD_CMD_H