#include "sd_diskio.h"

#include <string.h>

#include "lib/deprintf.h"
#include "lib/fatfs/ff.h"
#include "lib/fatfs/diskio.h"
#include "device/sd_cmd.h"

static DSTATUS stat = STA_NOINIT;
struct sd_cardinfo *cardinfo;

/*-----------------------------------------------------------------------*/
/* Get Drive Status                                                      */
/*-----------------------------------------------------------------------*/

DSTATUS SD_disk_status (void){

	return stat;
}



/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */
/*-----------------------------------------------------------------------*/

DSTATUS SD_disk_initialize(void) {
	
	enum sd_errcodes err;
	
	deprintf("BSP_SD_Init:");
	err = sd_init_card();
	
	if (SD_OK != err){
		stat = STA_NOINIT;
		deprintf(" ERR\n");
		return stat;
	}
	deprintf(" OK\n");
	
	cardinfo = sd_get_cardinfo();
	deprintf("-------------\n");
	deprintf("blocksize: %u\n", cardinfo->blocksize);
	deprintf("memory_capacity: %u bytes\n", cardinfo->memory_capacity);
	deprintf("block_nbr: %u\n", cardinfo->block_nbr);
	deprintf("-------------\n");
	
	stat = FR_OK;
	return stat;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */
/*-----------------------------------------------------------------------*/

DRESULT SD_disk_read (
	BYTE *buff,		/* Data buffer to store read data */
	DWORD sector,	/* Start sector in LBA */
	UINT count		/* Number of sectors to read */
)
{
	enum sd_errcodes err;
	err = sd_read_blocks(buff, sector, count, cardinfo->blocksize);
	
	if (SD_OK != err){
		return RES_ERROR;
	}
	
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */
/*-----------------------------------------------------------------------*/

DRESULT SD_disk_write (
	const BYTE *buff,	/* Data to be written */
	DWORD sector,		/* Start sector in LBA */
	UINT count			/* Number of sectors to write */
)
{
	enum sd_errcodes err;
	err = sd_write_blocks(buff, sector, count, cardinfo->blocksize);
	
	if (SD_OK != err){
		return RES_ERROR;
	}
	
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */
/*-----------------------------------------------------------------------*/

DRESULT SD_disk_ioctl (
	BYTE cmd,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	if(stat != FR_OK){
		return RES_ERROR;
	}
	
	switch(cmd){
	
		case CTRL_SYNC: //device has finished pending write process?
			return RES_OK; //ist imemr fertig, weil polling IO
			
		case GET_SECTOR_COUNT:
			memcpy(buff, &cardinfo->block_nbr, sizeof(DWORD));
			return RES_OK;
			
		case GET_SECTOR_SIZE:{
			WORD size = cardinfo->blocksize;
			memcpy(buff, &size, sizeof(WORD));
			return RES_OK;}
			
		case GET_BLOCK_SIZE:{
			DWORD ret = 1;
			memcpy(buff, &ret, sizeof(DWORD));
			return RES_OK;}
			
		case CTRL_TRIM:
			return RES_OK;
			
		default:
			return RES_PARERR;
	}
}

