#include "sd_spi.h"

#include "periph/spi.h"
#include "periph/rcc.h"
#include "periph/gpio.h"

void sd_spi_init(uint8_t clkdiv){
  
	// SCK Pin Init
	rcc_enable_clock(SPI_CK_PORT_RCC);
	gpio_initpin(SPI_CK_PORT, SPI_CK_PIN, AF_PP, GPIO_HS, SPI_CK_AF);
	
	// MOSI Pin Init
	rcc_enable_clock(SPI_MOSI_PORT_RCC);
	gpio_initpin(SPI_MOSI_PORT, SPI_MOSI_PIN, AF_PP, GPIO_HS, SPI_MOSI_AF);
	
	// MISO Pin Init
	rcc_enable_clock(SPI_MISO_PORT_RCC);
	gpio_initpin(SPI_MISO_PORT, SPI_MISO_PIN, AF_PP, GPIO_HS, SPI_MISO_AF);
	
	// CS Pin Init
	rcc_enable_clock(SPI_CS_PORT_RCC);
	gpio_initpin(SPI_CS_PORT, SPI_CS_PIN, GPO_PP, GPIO_HS, 0);
	sd_spi_setcs(1);
	
	// SPI Init
	rcc_enable_clock(SPI_RCC);
	spi_init_master(
		SPI_NBR, 	//SPI Base
		0, 			//MSB first
		clkdiv,		//CLK div
		1,			//CPOL -> 1:CK to 1 when idle
		1);			//CPHA -> 1:The second clock transition is the first data capture edge
}

void sd_spi_setcs(uint8_t highlow){ 
	gpio_onoff(SPI_CS_PORT, SPI_CS_PIN, highlow);
}

void sd_spi_read(uint8_t *rx, uint16_t len){
	spi_r_bytes(SPI_NBR, len, 0xFF, rx);
}

void sd_spi_write(const uint8_t *tx, uint16_t len){
	spi_w_bytes(SPI_NBR, len, (uint8_t*)tx);
}

uint8_t sd_spi_iobyte(uint8_t byte){
	return spi_rw_byte(SPI_NBR, byte);
}
