#ifndef SD_SPI_H
#define SD_SPI_H

#include <stdint.h>

#include "periph/spi.h"
#include "periph/rcc.h"
#include "periph/gpio.h"

#define SPI_NBR				SPI1_INDEX
#define SPI_RCC 			RCC_SPI1
#define SPI_CLKDIV_LOW		BR_DIV256	//Taktteiler der ein Takt zwischen 100 - 400kHz ausgibt (Card Init)
#define SPI_CLKDIV_HIGH		BR_DIV4		//Taktteiler fr spteren SD ZUgriff (max 25MHz)

#define SPI_CK_PORT_RCC		RCC_GPIOA
#define SPI_CK_PORT			PORTA
#define SPI_CK_PIN			5
#define SPI_CK_AF			AF5_SPI1_TO_2

#define SPI_MOSI_PORT_RCC	RCC_GPIOA
#define SPI_MOSI_PORT		PORTA
#define SPI_MOSI_PIN		7
#define SPI_MOSI_AF			AF5_SPI1_TO_2

#define SPI_MISO_PORT_RCC	RCC_GPIOA
#define SPI_MISO_PORT		PORTA
#define SPI_MISO_PIN		6
#define SPI_MISO_AF			AF5_SPI1_TO_2

#define SPI_CS_PORT_RCC		RCC_GPIOA
#define SPI_CS_PORT			PORTA
#define SPI_CS_PIN			3

/** \brief Initialisiert den SPI mit den GPIOs nach den obigen defines
*/
void sd_spi_init(uint8_t clkdiv);

/** \brief setzt den CS Pin
	\param highlow 0 -> CS LOW. 1 -> CS HIGH
*/
void sd_spi_setcs(uint8_t highlow);

/** \brief liest vom SPI Bus
	\param rx Pointer auf Buffer
	\param len Anzahl der zu lesenden Bytes
*/
void sd_spi_read(uint8_t *rx, uint16_t len);

/** \brief schreibt auf den SPI Bus
	\param tx Pointer auf Buffer
	\param len Anzahl der zu lesenden Bytes
*/
void sd_spi_write(const uint8_t *tx, uint16_t len);

/** \brief schreibt und liet ein Byte 
	\brief byte zu schreibendes Byte
	\return empfangenes Byte
*/
uint8_t sd_spi_iobyte(uint8_t byte);
   
#endif//SD_SPI