#include "gpio.h"

struct gpio {
	unsigned int MODER;
	unsigned int OTYPER;
	unsigned int OSPEEDER;
	unsigned int PUPDR;
	unsigned int IDR;
	unsigned int ODR;
	unsigned int BSRR;
	unsigned int LCKR;
	unsigned int AFRL;
	unsigned int AFRH;
};

int gpio_initpin(unsigned int gpio_base, uint8_t pin_nbr, enum initpin mode, enum pinspeed speed, enum altfuncs altfunc){
	
	volatile struct gpio * const gpio = (struct gpio *)gpio_base;
	
	if ((pin_nbr > 0xF) || (altfunc > 0xF)){
		return -1;
	}
	
	//Mode (Input, GP Out, Alternate function, Analog)
	uint32_t reg_mode = (mode & 0b11000) >> 3; //Bit 3 und 4 des enums sind die MODERx Bits
	gpio->MODER &= ~(0b11 << (pin_nbr*2));
	gpio->MODER |= (reg_mode << (pin_nbr*2));
	
	//Ausgangstreiber (Pushpull, Opendrain)
	uint32_t reg_ppod = (mode & 0b100) >> 2; //Bit 2 des enums ist OTx Bit
	gpio->OTYPER &= ~(1 << pin_nbr);
	gpio->OTYPER |= (reg_ppod << pin_nbr);
	
	//Slewrate (Lowspeed, Mediumspeed, Fastspeed, Highspeed)
	gpio->OSPEEDER &= ~(0b11 << (pin_nbr*2));
	gpio->OSPEEDER |= (speed << (pin_nbr*2));
	
	//Pullup, Pulldown
	uint32_t reg_pull = mode & 0b11; // Bit 0 und 1 des enums sind PUPDRx Bits
	gpio->PUPDR &= ~(0b11 << (pin_nbr*2));
	gpio->PUPDR |= (reg_pull << (pin_nbr*2));
	
	//Altfunc schreiben
	volatile unsigned int *afr = &(gpio->AFRL);
	
	if (pin_nbr > 7){
		afr = &(gpio->AFRH);
		pin_nbr -= 8;
	}
	
	*afr &= ~(0b1111 << (pin_nbr*4));
	*afr |= (altfunc << (pin_nbr*4));
	
	return 0;
}

void gpio_onoff(unsigned int gpio_base, uint8_t pin_nbr, uint8_t onoff){

	volatile struct gpio * const gpio = (struct gpio *)gpio_base;
	
	if (onoff){
		//Einschalten!
		gpio->BSRR = 1 << pin_nbr;
	} else {
		gpio->BSRR = 1 << (pin_nbr + 16);
	}
}

