#include "rcc.h"
#include "rcc_regdefs.h"

#define RCC_BASE_INT 0x40023800
static volatile struct rcc * const rcc = (struct rcc *)RCC_BASE_INT;

void rcc_set_noticed_clocks(unsigned int sysclk, unsigned int ahb, unsigned int apb1, unsigned int apb2){
	
	if (sysclk){
		clocks_s.sysclk = sysclk;
	}
	
	if (ahb){
		clocks_s.ahb = ahb;
	}
	
	if (apb1){
		clocks_s.apb1 = apb1;
	}
	
	if (apb2){
		clocks_s.apb2 = apb2;
	}
}

void rcc_get_noticed_clocks(unsigned int *sysclk, unsigned int *ahb, unsigned int *apb1, unsigned int *apb2){
	
	if (sysclk){
		*sysclk = clocks_s.sysclk;
	}
	
	if (ahb){
		*ahb = clocks_s.ahb;
	}
	
	if (apb1){
		*apb1 = clocks_s.apb1;
	}
	
	if (apb2){
		*apb2 = clocks_s.apb2;
	}
}

void rcc_enable_clock(enum clk_periph toenable){

	unsigned int bitnr = toenable & 0b00011111;
	unsigned int regnr = (toenable & 0b11000000) >> 6;
	regnr -= 1; //im enum steht AHB1, aber das ist index 0
	
	//passende basis raussuchen (AHB/APB)
	volatile unsigned int *reg = &(rcc->APB1ENR);
	if (toenable & 0b00100000){
		//war wohl doch AHB
		reg = &(rcc->AHB1ENR);
	}
	
	reg[regnr] |= (1 << bitnr);
}

int rcc_enable_hse(void){

	unsigned int counts;
	
	rcc->CR |= CR_HSEON;
	
	//Einschwingen pollen und nach 1000 Mal aufgeben
	for (counts = 0; counts < 1000; counts++){
		if (rcc->CR & CR_HSERDY){
			//Eingeschwungen!
			return 0;
		}
	}
	
	return -1;
}

int rcc_set_sysclk_source(enum sysclk_src src){

	unsigned int counts, reg;
	
	reg = rcc->CFGR & ~0b11;
	rcc->CFGR = reg | src;
	
	//Einschwingen pollen und nach 1000 Mal aufgeben
	for (counts = 0; counts < 1000; counts++){
		if (src == (rcc->CFGR & 0b11)){
			//Eingeschwungen!
			return 0;
		}
	}
	
	return -1;
}

void rcc_set_busprescaler(enum bus busen, enum busdiv divider){
	
	if (divider == RCC_BDIV_NONE){
		switch (busen){
			case RCC_APB1:
				rcc->CFGR &= ~(0b111 << CFGR_PPRE1_SHIFT);
				return;
			case RCC_APB2:
				rcc->CFGR &= ~(0b111 << CFGR_PPRE2_SHIFT);
				return;
			case RCC_AHB:
				rcc->CFGR &= ~(0b1111 << CFGR_HPRE_SHIFT);
				return;
		}
	}
	
	unsigned int regval = rcc->CFGR;
	
	switch (busen){
		case RCC_APB1:
			regval &= ~(0b111 << CFGR_PPRE1_SHIFT);
			regval |=  (0b100 << CFGR_PPRE1_SHIFT);
			regval |=  (divider << CFGR_PPRE1_SHIFT);
			break;
		case RCC_APB2:
			regval &= ~(0b111 << CFGR_PPRE2_SHIFT);
			regval |=  (0b100 << CFGR_PPRE2_SHIFT);
			regval |=  (divider << CFGR_PPRE2_SHIFT);
			break;
		case RCC_AHB:
			regval &= ~(0b1111 << CFGR_HPRE_SHIFT);
			regval |=  (0b1000 << CFGR_HPRE_SHIFT);
			regval |=  (divider << CFGR_HPRE_SHIFT);
			break;
	}
	
	rcc->CFGR = regval;
}

void rcc_set_mainpll(unsigned int pll_m, unsigned int pll_n, unsigned int pll_p, unsigned int pll_q, enum sysclk_src pll_src){
	
	unsigned int val = 0;
	
	val |= (pll_m & 0b111111) << PLLCFGR_PLLM_SHIFT;
	
	val |= (pll_n & 0b111111111) << PLLCFGR_PLLN_SHIFT;
	
	switch (pll_p) {
		case 2:
			pll_p = 0b00;
			break;
		case 4:
			pll_p = 0b01;
			break;
		case 6:
			pll_p = 0b10;
			break;
		case 8:
			pll_p = 0b11;
			break;
		default: 
			pll_p = 0b11;
			break;
	}
	val |= pll_p << PLLCFGR_PLLP_SHIFT;
	
	val |= (pll_q & 0b1111) << PLLCFGR_PLLQ_SHIFT;
	
	val |= (pll_src & 0b1) << PLLCFGR_PLLSRC_SHIFT;
	
	rcc->PLLCFGR = val;
	rcc->CR |= CR_PLLON;
	
}

int rcc_wait_mainpll_locked(void){

	unsigned int counts;
	
	//Einschwingen pollen und nach 1000 Mal aufgeben
	for (counts = 0; counts < 1000; counts++){
		if (rcc->CR & CR_PLLRDY){
			//Eingeschwungen!
			return 0;
		}
	}
	
	return -1;
}

void rcc_set_i2spll(unsigned int pll_n, unsigned int pll_r){
	
	unsigned int val = 0;
	val |= (pll_n & 0x1FF) << PLLI2SCFGR_PLLI2SN_SHIFT;
	val |= (pll_r & 0x7) << PLLI2SCFGR_PLLI2SR_SHIFT;
	
	rcc->CR &= ~CR_PLLI2SON;
	rcc->PLLI2SCFGR = val;
	rcc->CR |= CR_PLLI2SON;
}

int rcc_wait_i2spll_locked(void){

	unsigned int counts;
	
	//Einschwingen pollen und nach 1000 Mal aufgeben
	for (counts = 0; counts < 1000; counts++){
		if (rcc->CR & CR_PLLI2SRDY){
			//Eingeschwungen!
			return 0;
		}
	}
	
	return -1;
}

void rcc_set_mco1(enum mcodiv div, enum mco1_src src){

	unsigned int cfgr = rcc->CFGR;
	cfgr &= ~(0b111 << CFGR_MCO1PRE_SHIFT);
	cfgr &= ~(0b11 << CFGR_MCO1_SHIFT);
	cfgr |= div << CFGR_MCO1PRE_SHIFT;
	cfgr |= src << CFGR_MCO1_SHIFT;
	rcc->CFGR = cfgr;
}