#ifndef _RCC_H_
#define _RCC_H_

/** \brief enum Elemente zum Takteinschalten der Periph
	Bit 0 - 4 -> Bitnummer im Register, 
	Bit 5 -> AHB(1)/APB(0), 
	Bit 6, 7 -> AHB/APB Nummer,
*/
enum clk_periph {
	
	//AHB1
	RCC_GPIOA  	= 0b01100000,
	RCC_GPIOB  	= 0b01100001,
	RCC_GPIOC  	= 0b01100010,
	RCC_GPIOD  	= 0b01100011,
	RCC_GPIOE 	= 0b01100100,
	RCC_GPIOF  	= 0b01100101,
	RCC_GPIOG  	= 0b01100110,
	RCC_GPIOH  	= 0b01100111,
	RCC_GPIOI  	= 0b01101000,
	RCC_DMA1	= 0b01110101,
	RCC_DMA2	= 0b01110110,
	
	//APB1
	RCC_TIM2 	= 0b01000000,
	RCC_TIM3 	= 0b01000001,
	RCC_TIM4 	= 0b01000010,
	RCC_TIM5 	= 0b01000011,
	RCC_TIM6 	= 0b01000100,
	RCC_TIM7 	= 0b01000101,
	RCC_TIM12 	= 0b01000110,
	RCC_TIM13 	= 0b01000111,
	RCC_TIM14 	= 0b01001000,
	RCC_WWDG 	= 0b01001011,
	RCC_SPI2 	= 0b01001110,
	RCC_SPI3 	= 0b01001111,
	RCC_USART2 	= 0b01010001,
	RCC_USART3 	= 0b01010010,
	RCC_UART4 	= 0b01010011,
	RCC_UART5 	= 0b01010100,
	RCC_I2C1 	= 0b01010101,
	RCC_I2C2 	= 0b01010110,
	RCC_I2C3 	= 0b01010111,
	RCC_CAN1 	= 0b01011001,
	RCC_CAN2 	= 0b01011010,
	RCC_PWR 	= 0b01011100,
	RCC_DAC 	= 0b01011101,
	
	//APB2
	RCC_TIM1 	= 0b10000000,
	RCC_TIM8 	= 0b10000001,
	RCC_USART1 	= 0b10000100,
	RCC_USART6 	= 0b10000101,
	RCC_ADC1	= 0b10001000,
	RCC_ADC2	= 0b10001001,
	RCC_ADC3	= 0b10001010,
	RCC_SDIO 	= 0b10001011,
	RCC_SPI1 	= 0b10001100,
	RCC_SYSCFG 	= 0b10001110,
	RCC_TIM9 	= 0b10010000,
	RCC_TIM10 	= 0b10010001,
	RCC_TIM11 	= 0b10010010,
};

enum sysclk_src {
	RCC_HSI = 0b00,
	RCC_HSE = 0b01,
	RCC_PLL = 0b10,
};

enum bus {
	RCC_APB1 = 0,
	RCC_APB2 = 1,
	RCC_AHB  = 2,
};

enum busdiv {
	RCC_BDIV_NONE = 255,
	RCC_BDIV_2   = 0b000,
	RCC_BDIV_4   = 0b001,
	RCC_BDIV_8   = 0b010,
	RCC_BDIV_16  = 0b011,
	RCC_BDIV_64  = 0b100,
	RCC_BDIV_128 = 0b101,
	RCC_BDIV_256 = 0b110,
	RCC_BDIV_512 = 0b111,
};

enum mcodiv {
	RCC_MCO_DIV_NONE = 0,
	RCC_MCO_DIV_2 = 0b100,
	RCC_MCO_DIV_3 = 0b101,
	RCC_MCO_DIV_4 = 0b110,
	RCC_MCO_DIV_5 = 0b111,
};

enum mco1_src {
	MCO1_SRC_HSI = 0,
	MCO1_SRC_LSE = 1,
	MCO1_SRC_HSE = 2,
	MCO1_SRC_PLL = 3,
};

//Takte merken nach Initialisierungen der Taktdomne fr Inits der Periph
//0 bergeben -> Takt wird nicht gemerkt
void rcc_set_noticed_clocks(unsigned int sysclk, unsigned int ahb, unsigned int apb1, unsigned int apb2);

//Null -> Takt wird nicht zurck gegeben
void rcc_get_noticed_clocks(unsigned int *sysclk, unsigned int *ahb, unsigned int *apb1, unsigned int *apb2);

void rcc_enable_clock(enum clk_periph toenable);

// ret 0 bei Ok, sonst -1
int rcc_enable_hse(void);

// ret 0 bei Ok, sonst -1
int rcc_set_sysclk_source(enum sysclk_src src);

void rcc_set_busprescaler(enum bus busen, enum busdiv divider);

/*
pll_m: 2 - 63
pll_n: 192 - 432
pll_p: 2, 4, 6, 8
pll_q: 2 - 15
*/
void rcc_set_mainpll(unsigned int pll_m, unsigned int pll_n, unsigned int pll_p, unsigned int pll_q, enum sysclk_src pll_src);

// ret 0 bei locked, sonst -1
int rcc_wait_mainpll_locked(void);

/*
pll_n: 50 - 432
pll_r: 2 - 7
*/
void rcc_set_i2spll(unsigned int pll_n, unsigned int pll_r);

// ret 0 bei locked, sonst -1
int rcc_wait_i2spll_locked(void);

//MCO1 Ausgang einschalten mit div und mux
void rcc_set_mco1(enum mcodiv div, enum mco1_src src);

#endif//_RCC_H_