#ifndef _SPI_H_
#define _SPI_H_

#include <stdint.h>

#define SPI_USE_IRQ		1
#define SPI_USE_DMA		1

#define SPI1_INDEX 0
#define SPI2_INDEX 1
#define SPI3_INDEX 2

#define SPI1_BASE_INT 0x40013000
#define SPI2_BASE_INT 0x40003800
#define SPI3_BASE_INT 0x40003C00

#if SPI_USE_DMA
	#include "periph/dma.h"
#endif

//*************************************** SPI Abteilung **********************************************

enum spi_clkdiv {
	BR_DIV2 = 0,
	BR_DIV4 = 1,
	BR_DIV8 = 2,
	BR_DIV16 = 3,
	BR_DIV32 = 4,
	BR_DIV64 = 5,
	BR_DIV128 = 6,
	BR_DIV256 = 7,
};

void spi_init_master(
	unsigned int spi_base, 	//SPI Base
	unsigned int lsbfirst, 	//LSB first = 1
	enum spi_clkdiv div,	//CLK div
	unsigned int cpol,		//CPOL
	unsigned int cpha);		//CPHA
	
//schreibt ein byte und liet ein byte (return)
uint8_t spi_rw_byte(unsigned int spi_base, uint8_t byte);

/** \brief liet und schreibt Daten auf den SPI Bus
	\param tx Daten die auf den SPI Bus geschrieben werden
	\param rx Daten die vom SPI Bus gelesen werden
	\param len Anzahl der zu bertragenden Bytes
*/
void spi_rw_bytes(unsigned int spi_base, unsigned int bytes, uint8_t *tx, uint8_t *rx);

/** \brief schreibt Daten auf den SPI Bus
	\param tx Daten die auf den SPI Bus geschrieben werden
	\param len Anzahl der zu bertragenden Bytes
*/
void spi_w_bytes(unsigned int spi_base, unsigned int bytes, uint8_t *tx);

/** \brief liet Daten auf den SPI Bus
	\param idle_byte Byte das wiederholt geschrieben wird (zb 0xFF als SD Card IDLE)
	\param rx Daten die vom SPI Bus gelesen werden
	\param len Anzahl der zu bertragenden Bytes
*/
void spi_r_bytes(unsigned int spi_base, unsigned int bytes, uint8_t idle_byte, uint8_t *rx);

//Callback reinhngen fr "DMA hat alle Kanle durch"
//SPI Callback bekommt SPI Statusregister -> void callb_spi(unsigned int statreg, void *callb_userdata);
#if SPI_USE_DMA
int spi_dmamode(
	unsigned int spi_base, 							//Welcher SPI
	uint8_t *tx, 									//SPI Sendebuffer
	uint8_t *rx,									//SPI Empfangsbuffer
	enum dma_periph stream_tx,						//DMA Sendestream
	enum dma_periph stream_rx,						//DMA Empfangsstream
	uint16_t buflen,								//Anzahl der zu bertragenden Daten
	void (*callb_dma)(unsigned int, void*), 		//IRQ Handler DMA
	void *callb_data_dma, 							//IRQ Handler DMA Daten
	void (*callb)(unsigned int, void*), 			//IRQ Handler SPI Err
	void *callb_data);								//IRQ Handler SPI Err Daten
#endif

//*************************************** I2S Abteilung **********************************************

enum SPI_I2S_MODE {
	SPI_I2S_MODE_SLAVE_TX = 0x0,
	SPI_I2S_MODE_SLAVE_RX = 0x1,
	SPI_I2S_MODE_MASTER_TX = 0x2,
	SPI_I2S_MODE_MASTER_RX = 0x3,
};

enum SPI_I2S_STD {
	SPI_I2S_STD_I2S = 0x0,
	SPI_I2S_STD_LEFT_JUSTIFY = 0x1,
	SPI_I2S_STD_RIGHT_JUSTIFY = 0x2,
	SPI_I2S_STD_PCM = 0x3,
};

enum SPI_I2S_DATLEN {
	SPI_I2S_DATLEN_16BIT = 0x0,
	SPI_I2S_DATLEN_24BIT = 0x1,
	SPI_I2S_DATLEN_32BIT = 0x2,
};

#if SPI_USE_DMA
void spi_i2s_init(
	unsigned int spi_base, 		//Welcher SPI wird als I2S genutzt
	enum SPI_I2S_MODE i2smode, 	//senden/empfangen; master/slave
	enum SPI_I2S_STD standard, 	//I2s; left/tight
	enum SPI_I2S_DATLEN datlen,	//Datenlnge
	unsigned int mcko_enable,	//Master Clock Out
	enum dma_periph peri,		//Angabe der Peripherie
	unsigned int mem_addr0,
	unsigned int mem_addr1,
	uint16_t data_items_nbr,	//Anzahl der zu bertragenden Elemente
	void (*callb)(unsigned int, void*),	//Callback bei auftretendem IRQ
	void *callb_data						//Daten fr den Callback
	);
#endif

#endif//_SPI_H_