#ifndef _SPI_REGDEFS_H_
#define _SPI_REGDEFS_H_

struct spi {
	unsigned int CR1;
	unsigned int CR2;
	unsigned int SR;
	unsigned int DR;
	unsigned int CRCPR;		//CRC polynomial register
	unsigned int RXCRCR;	//RX CRC register
	unsigned int TXCRCR;	//TX CRC register
	unsigned int I2SCFGR;	//I2S configuration register
	unsigned int I2SPR;		//I2S prescaler register
};

//*************************************** Register CR1 **********************************************

enum reg_cr1 {
	CR1_BIDIMODE = 1 << 15,	//Bidirectional data mode enable
	CR1_BIDIOE = 1 << 14,	//Output enable in bidirectional mode
	CR1_CRCEN = 1 << 13,	//Hardware CRC calculation enable
	CR1_CRCNEXT = 1 << 12,	//CRC transfer next
	CR1_DFF = 1 << 11,		//Data frame format
	CR1_RXONLY = 1 << 10,	//This bit combined with the BIDImode bit selects the direction of transfer in 2-line unidirectional mode.
	CR1_SSM = 1 << 9,		//Software slave management
	CR1_SSI = 1 << 8,		//Internal slave select
	CR1_LSBFIRST = 1 << 7,	//Frame format
	CR1_SPE = 1 << 6,		//SPI enable
	CR1_BR_SHIFT = 3,		//Baud rate control
	CR1_MSTR = 1 << 2,		//Master selection
	CR1_CPOL = 1 << 1,		//Clock polarity
	CR1_CPHA = 1 << 0,		//Clock phase
};

//*************************************** Register CR2 **********************************************

enum reg_CR2 {
	CR2_TXEIE = 1 << 7,		//Tx buffer empty interrupt enable
	CR2_RXNEIE = 1 << 6,	//RX buffer not empty interrupt enable
	CR2_ERRIE = 1 << 5,		//Error interrupt enable
	CR2_FRF = 1 << 4,		//Frame format
	CR2_SSOE = 1 << 2,		//SS output enable
	CR2_TXDMAEN = 1 << 1,	//Tx buffer DMA enable
	CR2_RXDMAEN = 1 << 0,	//Rx buffer DMA enable
};

//*************************************** Register SR **********************************************

enum reg_SR {
	SR_TIFRFE = 1 << 8,		//TI frame format error
	SR_BSY = 1 << 7,		//Busy flag
	SR_OVR = 1 << 6,		//Overrun flag
	SR_MODF = 1 << 5,		//Mode fault
	SR_CRCERR = 1 << 4,		//CRC error flag
	SR_UDR = 1 << 3,		//Underrun flag
	SR_CHSIDE = 1 << 2,		//Channel side
	SR_TXE = 1 << 1,		//Transmit buffer empty
	SR_RXNE = 1 << 0,		//Receive buffer not empty
};

//*************************************** Register I2SCFGR *****************************************

enum reg_I2SCFGR {
	I2SCFGR_I2SMOD = 1 << 11,	//0 -> SPI; 1: I2S mode is selected
	I2SCFGR_I2SE = 1 << 10,		//1: I2S peripheral is enabled
	I2SCFGR_I2SCFG_SHIFT = 8,	//00: Slave - transmit; 01: Slave - receive; 10: Master - transmit; 11: Master - receive
	I2SCFGR_PCMSYNC = 1 << 7,	//frame synchronization: 0: Short; 1: Long
	I2SCFGR_I2SSTD_SHIFT = 4,	//I2S standard selection
	I2SCFGR_CKPOL = 1 << 3,		//1: I2S clock steady state is high level
	I2SCFGR_DATLEN_SHIFT = 1,	//
	I2SCFGR_CHLEN = 1 << 0,		//Channel length (number of bits per audio channel): 0 - 16Bit, 1 - 32Bit
};	

enum bits_I2SCFG {
	I2SCFG_SLAVE_TX = 0x0,
	I2SCFG_SLAVE_RX = 0x1,
	I2SCFG_MASTER_TX = 0x2,
	I2SCFG_MASTER_RX = 0x3,
};

enum bits_I2SSTD {
	I2SSTD_I2S = 0x0,
	I2SSTD_LEFT_JUSTIFY = 0x1,
	I2SSTD_RIGHT_JUSTIFY = 0x2,
	I2SSTD_PCM = 0x3,
};

enum bits_DATLEN {
	DATLEN_16BIT = 0x0,
	DATLEN_24BIT = 0x1,
	DATLEN_32BIT = 0x2,
};

//*************************************** Register I2SPR *******************************************

enum reg_I2SPR {
	I2SPR_MCKOE = 1 << 9,	//1: Master clock output is enabled
	I2SPR_ODD = 1 << 8,		//1: real divider value is = (I2SDIV * 2)+1
	I2SPR_I2SDIV_SHIFT = 0,	//I2SDIV [7:0] = 0 or I2SDIV [7:0] = 1 are forbidden values.
};

#endif//_SPI_REGDEFS_H_